{********************************************************************
TSHADER component
for Delphi 2.0, 3.0, 4.0, 5.0 C++Builder 1.0,3.0,4.0
version 1.1

written by 
      TMS Software
      copyright  1998-1999
      Email : info@tmssoftware.com
      Web : http://www.tmssoftware.com
********************************************************************}

unit Shader;

{$IFDEF WIN32}
 {$R SHADR32.DCR}
{$ELSE}
 {$R SHADR16.DCR}
{$ENDIF}

{$IFDEF VER110}
 {$ObjExportAll On}
{$ENDIF}

{$IFDEF VER125}
 {$ObjExportAll On}
{$ENDIF}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls;

type
  TShader = class(TPanel)
  private
    TFromColor:TColor;
    TToColor:TColor;
    TSteps:word;
    TDirection:boolean;
    procedure SetFromColor(Color:TColor);
    procedure SetToColor(Color:TColor);
    procedure SetSteps(w:word);
    procedure SetDirection(b:boolean);
  protected
    procedure Paint; override;
    { Protected declarations }
  public
    constructor Create(AOwner:TComponent); override;
    { Public declarations }
  published
    property FromColor:TColor read TFromColor write SetFromCOlor default clBlue;
    property ToColor:TColor read TToColor write SetToColor default clBlack;
    property Steps:word read TSteps write SetSteps default 20;
    property Direction:boolean read TDirection write SetDirection;
    { Published declarations }
  end;

procedure Register;

implementation

constructor TShader.Create(AOwner:TComponent);
begin
 inherited Create(AOwner);
 TSteps:=20;
 TDirection:=false;
 TFromColor:=clBlue;
 TToColor:=clBlack;
end;

procedure TShader.SetFromColor(Color:TColor);
begin
 TFromColor:=Color;
 Invalidate;
end;

procedure TShader.SetToColor(Color:TColor);
begin
 TToColor:=Color;
 Invalidate;
end;

procedure TShader.SetSteps(w:word);
begin
 TSteps:=w;
 Invalidate;
end;

procedure TShader.SetDirection(b:boolean);
begin
 TDirection:=b;
 Invalidate;
end;

procedure TShader.Paint;
var
 diffr,stepr,startr,endr:longint;
 diffg,stepg,startg,endg:longint;
 diffb,stepb,startb,endb:longint;
 rstepr:real;
 rstepg:real;
 rstepb:real;
 rstepw:real;
 steps,i,stepw:word;

begin
 if TSteps=0 then TSteps:=1;
 startr:=(TFromColor and $0000ff);
 startg:=(TFromColor and $00ff00) shr 8;
 startb:=(TFromColor and $ff0000) shr 16;
 endr:=(TToColor and $0000ff);
 endg:=(TToColor and $00ff00) shr 8;
 endb:=(TToColor and $ff0000) shr 16;

 diffr:=endr-startr;
 diffg:=endg-startg;
 diffb:=endb-startb;

 rstepr:=diffr/Tsteps;
 rstepg:=diffg/Tsteps;
 rstepb:=diffb/Tsteps;

 if TDirection then
  rstepw:=width/TSteps
 else
  rstepw:=height/TSteps;

 with Canvas do
  begin
   for i:=0 to Tsteps-1 do
    begin
     endr:=startr+round(rstepr*i);
     endg:=startg+round(rstepg*i);
     endb:=startb+round(rstepb*i);
     stepw:=round(i*rstepw);
     pen.color:=endr + (endg shl 8) + (endb shl 16);
     brush.color:=pen.color;

     if TDirection then
      fillrect(Rect(stepw - 2, 0, stepw+round(rstepw)+1, height  ))
     else
      fillrect(Rect(0, stepw -2 ,width, stepw+round(rstepw)+ 1));

    end;

  end;
end;


procedure Register;
begin
  RegisterComponents('TMS', [TShader]);
end;

end.
